/** interrupt.c
 *
 *  Author: J. Krauth
 *  Creation: 23.09.2015
 *  $LastChangedDate: 2015-01-12 17:26:28 +0100 (Mo, 12. Jan 2015) $
 *  $LastChangedRevision: 1381 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#include "board_data.h"
#include "private.h"

#ifndef CONFIG_XENOMAI
#include <linux/interrupt.h>
#endif
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/signal.h>

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

extern int i_PCIE040_TimerWatchdogInterruptHandler (struct pci_dev *pdev);

extern int i_PCIE040_TemperatureInterruptHandler (struct pci_dev *pdev);

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

void v_PCIE040_DispatchInterrupt (struct pci_dev *pdev, uint32_t interruptSource, uint32_t interruptMask, uint32_t * value, uint32_t count)
{
    /* Send the interrupt details to the registered kernel mode interrupt callback */
    /* TODO: use a list instead of a single pointer to allow registering more than one callback */
    if (PCIE040_PRIVDATA(pdev)->irqCallback != NULL)
        (PCIE040_PRIVDATA(pdev)->irqCallback)(pdev, 0, interruptSource, interruptMask, value, count);

#ifndef CONFIG_XENOMAI
    /* Send the SIGIO signal to registered processes to notify that an interrupt occurred */
    if (PCIE040_PRIVDATA(pdev)->asyncQueue != NULL)
        kill_fasync(&(PCIE040_PRIVDATA(pdev)->asyncQueue), SIGIO, POLL_IN);
#endif
}

//--------------------------------------------------------------------------------------

#ifdef CONFIG_XENOMAI
static int pcie040_irq_handler(rtdm_irq_t *irq_context)
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
static irqreturn_t pcie040_irq_handler(int irq, void *dev_id)
#else
static irqreturn_t pcie040_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
#endif
{
    int irqCount = 0;
#ifdef CONFIG_XENOMAI
    struct pci_dev *pdev = rtdm_irq_get_arg(irq_context, struct pci_dev);
#else
    struct pci_dev *pdev = (struct pci_dev *)dev_id;
#endif

    irqCount = i_PCIE040_TimerWatchdogInterruptHandler (pdev);
    irqCount += i_PCIE040_TemperatureInterruptHandler (pdev);

#ifdef CONFIG_XENOMAI
    return (irqCount > 0 ? RTDM_IRQ_HANDLED : RTDM_IRQ_NONE);
#else
    return (irqCount > 0 ? IRQ_HANDLED : IRQ_NONE);
#endif
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_SetBoardIntRoutine);
int i_PCIE040_SetBoardIntRoutine(struct pci_dev *pdev, pcie040_irqCallback irqCallback)
{
    int ret = 0;
    PCIE040_PRIVDATA(pdev)->irqCallback = irqCallback;

    print(KERN_INFO MODULE_NAME ": registering handler for IRQ %d\n", pdev->irq);
#ifdef CONFIG_XENOMAI
    ret = rtdm_irq_request(&PCIE040_PRIVDATA(pdev)->irqHandle,
                           pdev->irq,
                           &pcie040_irq_handler,
                           RTDM_IRQTYPE_SHARED,
                           MODULE_NAME,
                           pdev);
#else
    ret = request_irq(pdev->irq, pcie040_irq_handler, IRQF_SHARED, MODULE_NAME, pdev);
#endif

    if (ret == 0)
    {
        /* Enable the x86 interrupt */
		uint32_t value = ioread32 (PCIE040_PRIVDATA(pdev)->bar0 + 0x68);
		value |= (1 << 11) | (1 << 8);
		iowrite32 (value, PCIE040_PRIVDATA(pdev)->bar0 + 0x68);

        PCIE040_PRIVDATA(pdev)->irqRequested = 1;
    }
    return ret;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_ResetBoardIntRoutine);
int i_PCIE040_ResetBoardIntRoutine(struct pci_dev *pdev)
{
    PCIE040_PRIVDATA(pdev)->irqCallback = NULL;

    if (PCIE040_PRIVDATA(pdev)->irqRequested != 0)
    {
        PCIE040_PRIVDATA(pdev)->irqRequested = 0;

        print(KERN_INFO MODULE_NAME ": disabling handler for IRQ %d\n", pdev->irq);
#ifdef CONFIG_XENOMAI
        return rtdm_irq_free(&PCIE040_PRIVDATA(pdev)->irqHandle);
#else
        free_irq(pdev->irq, pdev);
        return 0;
#endif
    }
    return 1;
}

//--------------------------------------------------------------------------------------

//EXPORT_SYMBOL(i_PCIE040_TestInterrupt);
//int i_PCIE040_TestInterrupt(struct pci_dev *pdev, uint32_t *interruptSource, uint32_t *interruptMask, uint32_t *latchedValue)
//{
//    *interruptSource = PCIE040_PRIVDATA(pdev)->interruptSource;
//    *interruptMask   = PCIE040_PRIVDATA(pdev)->interruptMask;
//    *latchedValue    = PCIE040_PRIVDATA(pdev)->latchedValue;
//
//    PCIE040_PRIVDATA(pdev)->interruptSource = 0;
//    PCIE040_PRIVDATA(pdev)->interruptMask   = 0;
//    PCIE040_PRIVDATA(pdev)->latchedValue    = 0;
//
//    return (*interruptMask != 0 ? pdev->irq : 1);
//}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

