/** pci.c
 *
 *  Author: J. Krauth
 *  Creation: 22.09.2015
 *  $LastChangedDate: 2014-12-16 17:41:40 +0100 (Di, 16. Dez 2014) $
 *  $LastChangedRevision: 1332 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#include "board_data.h"
#include "private.h"
#include <linux/version.h>
#include <linux/fs.h>

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

static int _registered = 0;
static int _major = 0;
static struct pci_dev *_firstBoard = NULL;
#ifndef CONFIG_XENOMAI
static struct class *_class = NULL;
#endif
//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifdef CONFIG_XENOMAI
	static struct rtdm_driver rtdmdrv_deflault = {

		.profile_info		= RTDM_PROFILE_INFO(MODULE_NAME,
												RTDM_CLASS_TESTING,
												RTDM_SUBCLASS_GENERIC,
												1),

		.device_count = 1,
		.device_flags = RTDM_NAMED_DEVICE,

		.ops = {
				.open = pcie040_open_rt,
				.close = pcie040_close_rt,
				.ioctl_rt = pcie040_ioctl_rt,
				.ioctl_nrt = pcie040_ioctl_rt,
		},
	};
#endif

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(p_PCIE040_GetPciDevById);
struct pci_dev *p_PCIE040_GetPciDevById(unsigned int id)
{
    struct pci_dev *board;
    for (board = _firstBoard; board != NULL; board = PCIE040_PRIVDATA(board)->nextBoard)
        if (PCIE040_PRIVDATA(board)->id == id)
            return board;
    return NULL;
}

//--------------------------------------------------------------------------------------

static int pcie040_pci_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
{
    int ret;
    pcie040BoardData_t *boardData;

#ifdef CONFIG_XENOMAI
    struct rtdm_device *rtdmdev;
#endif

    ret = pci_enable_device (pdev);
    if (unlikely (ret != 0))
    {
        print (KERN_ERR MODULE_NAME ": pci_enable_device failed\n");
        goto end_pci_enable_device;
    }

#ifdef CONFIG_XENOMAI
    /* Allocate memory to store the RTDM device */
    rtdmdev = kmalloc (sizeof (struct rtdm_device), GFP_ATOMIC);
    if (unlikely (rtdmdev == NULL))
    {
        print (KERN_CRIT MODULE_NAME ": can't allocate memory for new board %s\n", pci_name(pdev));
        ret = -ENOMEM;
        goto end_kmalloc_rtdmdevc;
    }

    /* The PCI device driver custom data has to point on the new RTDM device */
    pci_set_drvdata (pdev, rtdmdev);
#endif

    /* Allocate memory to store private data for this board */
    boardData = kmalloc (sizeof (pcie040BoardData_t), GFP_ATOMIC);
    if (unlikely (boardData == NULL))
    {
        print (KERN_CRIT MODULE_NAME ": can't allocate memory for new board %s\n", pci_name(pdev));
        ret = -ENOMEM;
        goto end_malloc_boardData;
    }

    memset (boardData, 0, sizeof (pcie040BoardData_t));

#ifndef CONFIG_XENOMAI
    pci_set_drvdata (pdev, boardData);
#else
    /* Keep a pointer on pdev in the RTDM structure custom data */
    boardData->pdev = pdev;

    /* The RTDM device get a pointer on the board data structure */
    rtdmdev->device_data = boardData;
#endif

    /* Lock BAR resources */
    ret = pci_request_regions(pdev, MODULE_NAME);
    if (unlikely(ret != 0))
    {
        print (KERN_ERR MODULE_NAME ": pci_request_regions failed\n");
        goto end_pci_request_regions;
    }

    /* Map BAR 0 */
    boardData->bar0 = ioremap (pdev->resource[0].start, pci_resource_len(pdev, 0));

    /* Add this board to the list */
    if (unlikely(_firstBoard == NULL))
    {
        _firstBoard = pdev;
        boardData->id = 0;
    }
    else
    {
        struct pci_dev *board = _firstBoard;
        while (PCIE040_PRIVDATA(board)->nextBoard != NULL)
            board = PCIE040_PRIVDATA(board)->nextBoard;

        PCIE040_PRIVDATA(board)->nextBoard = pdev;
        boardData->prevBoard = board;
        boardData->id = PCIE040_PRIVDATA(board)->id + 1;
    }

#ifndef CONFIG_XENOMAI
    /* Create a device file */
    if (!IS_ERR(_class))
        if (IS_ERR(device_create(_class, NULL, MKDEV(_major, boardData->id), NULL, MODULE_NAME "_%d", boardData->id)))
            printk(KERN_WARNING MODULE_NAME ": device_create failed\n");    
#else    
    /* Create the RTDM device */
    rtdmdev->label = "pcie040_%d";
    rtdmdev->driver = &rtdmdrv_deflault;

	ret = rtdm_dev_register (rtdmdev);
	if (unlikely (ret != 0))
	{
		print (KERN_INFO MODULE_NAME ": rtdm_dev_register error\n");
		goto end_rtdm_dev_register;
	}
#endif

    /* Print board infos */
	print (KERN_INFO MODULE_NAME ": board %s added (id %d)\n", pci_name(pdev), boardData->id);
    
    /* Start the temperature conversion */
    outl (0x80000, pdev->resource[3].start + 8);

    return 0;

end_rtdm_dev_register:
	pci_release_regions (pdev);

end_pci_request_regions:
	if (boardData)
		kfree (boardData);

end_malloc_boardData:
#ifdef CONFIG_XENOMAI
	if (rtdmdev)
		kfree (rtdmdev);
#endif

end_kmalloc_rtdmdevc:
	pci_disable_device (pdev);

end_pci_enable_device:

	return ret;
}

static void pcie040_pci_remove(struct pci_dev *pdev)
{
    unsigned int id = (unsigned int)-1;

    /* Remove the board from the board list */
    pcie040BoardData_t *boardData = PCIE040_PRIVDATA(pdev);
    if (boardData != NULL)
    {
        struct pci_dev *nextBoard = boardData->nextBoard;
        struct pci_dev *prevBoard = boardData->prevBoard;

        if (nextBoard != NULL)
            PCIE040_PRIVDATA(nextBoard)->prevBoard = prevBoard;

        if (prevBoard != NULL)
            PCIE040_PRIVDATA(prevBoard)->nextBoard = nextBoard;

        id = boardData->id;
    }
#ifdef CONFIG_XENOMAI
    {
    	struct rtdm_device *rtdmdev  = (struct rtdm_device *) pci_get_drvdata(pdev);

    	rtdm_dev_unregister (rtdmdev);
    	kfree (rtdmdev);
    }
#endif
    /* Do board dependent cleanup */
    i_PCIE040_ResetBoardIntRoutine (pdev);

    /* Release the PCI device */
    pci_release_regions (pdev);
    pci_disable_device (pdev);

    /* Free private data */
    if (boardData != NULL)
        kfree(boardData);

    print (KERN_INFO MODULE_NAME ": board %s removed (id %d)\n", pci_name(pdev), id);
};

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

extern struct file_operations pcie040_fops;

#ifdef DEFINE_PCI_DEVICE_TABLE
DEFINE_PCI_DEVICE_TABLE(pcie040_pci_table) =
#else
static struct pci_device_id pcie040_pci_table[] __devinitdata =
#endif
{
    {   /* APCI-040 */
        0x15B8,
        0x0301,
        PCI_ANY_ID,
        PCI_ANY_ID,
        0,
        0,
        0
    },
    { }
};

static struct pci_driver pcie040_pci_driver = {
    .name     = MODULE_NAME,
    .id_table = pcie040_pci_table,
    .probe    = pcie040_pci_probe,
    .remove   = pcie040_pci_remove
};

int __init pcie040_pci_init(void)
{
	int ret = 0;

    _major = register_chrdev (0, MODULE_NAME, &pcie040_fops);
	if (unlikely (_major < 0))
	{
		print (KERN_INFO MODULE_NAME ": register_chrdev error\n");
		goto error;
	}

#ifndef CONFIG_XENOMAI
    _class = class_create (THIS_MODULE, MODULE_NAME);
	if (unlikely (_class == NULL))
	{
		print (KERN_INFO MODULE_NAME ": class_create error\n");
		goto error;
	}
#endif

	ret = pci_register_driver (&pcie040_pci_driver);
	if (unlikely (ret < 0))
	{
		print (KERN_INFO MODULE_NAME ": pci_register_driver error\n");
		goto error;
	}

	_registered = 1;

    goto end;

error:

	if (_registered != 0)
		pci_unregister_driver (&pcie040_pci_driver);

#ifndef CONFIG_XENOMAI
	if (_class != NULL)
		class_destroy(_class);
#endif

	if (_major != 0)
		unregister_chrdev(_major, MODULE_NAME);

	return 1;

end:
	print (KERN_INFO MODULE_NAME ": PCI initialized (major number is %d)", _major);

	return 0;
}

void __exit pcie040_pci_cleanup(void)
{
    if (_registered != 0)
    {
        pci_unregister_driver(&pcie040_pci_driver);
        _registered = 0;
    }

#ifndef CONFIG_XENOMAI
	if (_class != NULL)
		class_destroy(_class);
#endif

    if (_major != 0)
    {
        unregister_chrdev(_major, MODULE_NAME);
        _major = 0;
    }
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
