#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/syscall.h>

#include <pcie040.h>

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

volatile uint8_t quit = 0;

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

void quit_signal (int sig)
{
	quit = 1;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

void * task_body (void *arg)
{
	int ret = 0;
	uint8_t last[4] = {0};
	uint8_t *index = (uint8_t *)arg;
	uint8_t value = 0;
	uint8_t status = 0;
	uint8_t softwareTriggerStatus = 0;
	uint8_t hardwareTriggerStatus = 0;

	struct timespec tp;
	char buffer [80];
	struct itimerspec new_value;
	struct sigaction action;
	struct sigevent sigev;
	sigset_t set;
	int signum = SIGALRM;
	timer_t timer;

	sigev.sigev_notify = SIGEV_THREAD_ID;
	sigev.sigev_notify_thread_id = syscall(SYS_gettid);
	sigev.sigev_signo = signum;

	sigemptyset(&set);
	sigaddset(&set, signum);
	sigprocmask(SIG_BLOCK, &set, NULL);

	if (timer_create (CLOCK_MONOTONIC, &sigev, &timer) == -1)
	{
		perror ("timer_create");
		goto end;
	}

	new_value.it_interval.tv_sec = 0;
	new_value.it_interval.tv_nsec = 500000000llu;	// 500 ms
	new_value.it_value.tv_sec = 0;
	new_value.it_value.tv_nsec = 500000000llu;		// 500 ms

	if (timer_settime (timer, 0, &new_value, NULL) == -1)
	{
		perror ("timer_settime");
		goto end;
	}

    while (!quit)
    {
		/* wait for signal */
		if (sigwait (&set, &signum) == -1)
		{
			perror ("sigwait");
			goto end;
		}

		if ((ret = i_PCIE040_TimerWatchdogReadStatusAndValue (0, *index, &value, &status, &softwareTriggerStatus, &hardwareTriggerStatus)) != 0)
		{
			if (ret < 0)
				perror ("i_PCIE040_TimerWatchdogReadStatusAndValue");
			else
				printf("i_PCIE040_TimerWatchdogReadStatusAndValue error %i\n", ret);

			quit = 1;
		}

		if ((value != last[0]) || (status != last[1]) || (softwareTriggerStatus != last[2]) || (hardwareTriggerStatus != last[3]))
		{
			printf ("The watchdog status is %x, value is %u, soft trigger is %x, hard trigger is %x \n", status, value, softwareTriggerStatus, hardwareTriggerStatus);

			/* Use to update the display only when a value changes */
			last[0] = value;
			last[1] = status;
			last[2] = softwareTriggerStatus;
			last[3] = hardwareTriggerStatus;

			/* The watchdog runs down, we can quit the sample */
			if (status)
				quit = 1;
		}
    }

	end:
		quit = 1;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int main (int argc, char *argv[])
{
	int ret = 0;
	pthread_t thread;
	uint8_t boardCount = 0;

	uint8_t index = 10;
    uint8_t mode = 0;						// 0: Watchdog, 1: timer
    uint8_t timeUnit = 3;					// 1: us, 2: ms, 3: s, 4: min
    uint8_t timeValue = 10;					// 1 to 255
    uint8_t useGate = 0;					// 0: no, 1: yes
    uint8_t gateLevel = 0;					// 1: low active, 2: high active
    uint8_t useHardwareTrigger = 0;			// 0: no, 1: yes
    uint8_t hardwareTriggerLevel = 0;		// 1: low active, 2: high active, 3: low and high active
    uint8_t warningRelayMode = 2;			// 0: no, 1: Mode 1, 2: Mode 2
    uint8_t warningRelayDelayTimeUnit = 0;	// 1: us, 2: ms, 3: s
    uint8_t warningRelayDelayTimeValue = 0;	// 1 to 255
    uint8_t useResetRelay = 0;				// 0: no, 1: yes
    uint8_t resetRelayMode = 0;				// 0: Reset relay is set when the watchdog has run down
    										// 1: Reset relay is reseted when the watchdog has run down
    uint8_t useInterrupt = 0;

	if ((ret = i_PCIE040_GetNumberOfBoards (&boardCount)) != 0)
	{
		printf("i_PCIE040_GetNumberOfBoards error %i\n", ret);
		exit(1);
	}

	if (boardCount == 0)
	{
		printf ("No board found\n");
		exit(1);
	}

	printf ("Number of boards: %i\n", boardCount);

    do
    {
        char inputBuffer[32];
        printf("Select the watchdog you would like to use (0 to 6): ");
        fflush(stdout);
        if (fgets(inputBuffer, sizeof(inputBuffer), stdin) != NULL && strlen(inputBuffer) > 1)
        	index = (unsigned int)atoi(inputBuffer);
    } while (index > 6);

	/* Manage Ctrl + C */
	signal (SIGINT, quit_signal);

	if ((ret = i_PCIE040_TimerWatchdogInitAndStart (0,
													index,
													mode,
													timeUnit,
													timeValue,
													useGate,
													gateLevel,
													useHardwareTrigger,
													hardwareTriggerLevel,
													warningRelayMode,
													warningRelayDelayTimeUnit,
													warningRelayDelayTimeValue,
													useResetRelay,
													resetRelayMode,
													useInterrupt)) != 0)
	{
		if (ret < 0)
			perror ("i_PCIE040_TimerWatchdogInitAndStart");
		else
			printf("i_PCIE040_TimerWatchdogInitAndStart error %i\n", ret);

		goto end;
	}

	ret = pthread_create (&thread, NULL, task_body, (uint8_t *)&index);
	if (ret != 0)
	{
		perror("pthread_create\n");
		goto end;
	}

	/* Wait that the customer use the combination Ctrl + C to quit the sample */
	while (!quit)
		sleep (1);

end:
	quit = 1;

	if ((ret = i_PCIE040_TimerWatchdogStartStop (0, index, 0)) != 0)
	{
		if (ret < 0)
			perror ("i_PCIE040_TimerWatchdogStartStop");
		else
			printf("i_PCIE040_TimerWatchdogStartStop error %i\n", ret);
	}

	pthread_join(thread, NULL);

	/* Close the device */
	if ((ret = i_PCIE040_CloseBoard (0)) != 0)
		printf("i_PCIE040_CloseBoard error %i\n", ret);

	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
