/** pcie040.h
 *
 *  Author: J. Krauth
 *  Creation: 23.09.2015
 *  $LastChangedDate: 2015-02-04 16:34:40 +0100 (Mi, 04. Feb 2015) $
 *  $LastChangedRevision: 1394 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifndef __PCIE040_H__
#define __PCIE040_H__

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#include <inttypes.h>

/** \defgroup PCIE040_USER_MODE_API xPCIe-040 user mode API
 * @{
 */
/* @} */

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

/** \addtogroup General xPCIe-040 general functions
* \ingroup PCIE040_USER_MODE_API
 * @{
 *
 */

/** Get the number of boards.
 *
 * @param[out] boardCount         : Number of boards \n
 *
 * @return       0 : No board found \n
 *             > 0 : Number of boards \n
 *              -1 : Fail to get the number of boards
 */
int i_PCIE040_GetNumberOfBoards(uint8_t *boardCount);

/** Close the handle on the board.
 *
 * @param[out] boardId         : ID number of the device to close \n
 *
 * @return       0 : No board found \n
 *             < 0 : see close or rt_dev_close \n
 *              -1 : Fail to get the number of boards
 */
int i_PCIE040_CloseBoard(uint8_t boardId);

/* @} */

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

/** \addtogroup Temperature xPCIe-040 temperature functions
* \ingroup PCIE040_USER_MODE_API
 * @{
 *
 */

/** Permits to check if a temperature value is ready to be read on the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[out] eoc                       : End of conversion \n
 *                                         0: Temperature value is not ready  \n
 *                                         1: Temperature value is ready  \n
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 */
int i_PCIE040_TemperatureTestConversion (uint8_t boardId,
                                          uint8_t *eoc);

//--------------------------------------------------------------------------------------

/** Read the temperature value of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[out] value                     : Temperature value in °C
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 */
int i_PCIE040_TemperatureReadValue (uint8_t boardId,
                                     uint32_t *value);

//--------------------------------------------------------------------------------------

/** Permits to initialize and start the temperature warning of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] value                     : Temperature warning value in °C
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 *            2 : Temperature value out of range \n
 */
int i_PCIE040_TemperatureWarningInit (uint8_t boardId,
                                       uint32_t value);

//--------------------------------------------------------------------------------------

/** Permits to start / stop the temperature warning of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] cmd                        : 0: Stop \n
 *                                         1: start \n
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 *            2 : Error by the start / stop of the temperature warning \n
 */
int i_PCIE040_TemperatureWarningStartStop (uint8_t boardId,
                                            uint8_t cmd);

/* @} */

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

/** \addtogroup TimerWatchdog xPCIe-040 timer / watchdog functions
* \ingroup PCIE040_USER_MODE_API
 * @{
 *
 */

/** Permits to initialize, enable and trigger the timer/watchdog of the board xPCIe-040 (starts the watchdog immediately)
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] index                      : Timer/Watchdog index (0 to 6) \n
 * @param[in] mode                       : 0: WATCHDOG \n
 *                                         1: TIMER \n
 * @param[in] timeUnit                   : Time unit \n
 *                                         1: µs 2: ms 3: s 4: min \n
 * @param[in] timeValue                  : Time value (1 to 255)
 * @param[in] useGate                    : (Only available in timer mode) Define if the gate is used or not \n
 *                                         0: Gate not used \n
 *                                         1: Gate used \n
 * @param[in] gateLevel                  : (Only available in timer mode)Gate level selection \n
 *                                         1: If the hardware gate is enabled, it is active at "0" \n
 *                                         2: If the hardware gate is enabled, it is active at "1" \n
 * @param[in] useHardwareTrigger         : (Only available in watchdog mode) Define if the hardware trigger is used or not \n
 *                                         0: Hardware trigger not used \n
 *                                         1: Hardware trigger used \n
 * @param[in] hardwareTriggerLevel       : (Only available in watchdog mode) Hardware trigger level selection \n
 *                                         1: If the hardware trigger is used, it triggers from "1" to "0" \n
 *                                         2: If the hardware trigger is used, it triggers from "0" to "1" \n
 *                                         3: If the hardware trigger is used, it triggers from "0" to "1" and from "1" to "0" \n
 * @param[in] warningRelayMode            : (Only available in watchdog mode on watchdog 1 to 6) Define if the warning relay mode \n
 *                                         0 : Warning relay not used \n
 *                                         1 : warning mode 1 \n
 *                                             If index = 0, the warning relais goes high when the watchdog runs down and goes low after the specified delay\n
 *                                             If index > 0, the warning relais goes high when the watchdog runs down. It goes low when the watchdog is newly initialized\n
 *                                         2 : warning mode 2 \n
 *                                             The warning relais goes high when the watchdog starts and goes low when the watchdog runs down
 * @param[in] warningRelayDelayTimeUnit  : (Only available in watchdog mode) Delay time between the warning relay and the activation of the reset relay \n
 *                                         1: µs 2: ms 3: s \n
 * @param[in] warningRelayDelayTimeValue : (Only available in watchdog mode) Delay value (1 to 255) \n
 * @param[in] useResetRelay              : (Only available for the first watchdog) Define if the reset delay is used or not \n
 *                                         0: Reset relay not used \n
 *                                         1: Reset relay used \n
 * @param[in] resetRelayMode             : (Only available for the first watchdog) Define the reset relay mode \n
 *                                         0: Reset relay is set when watchdog has run down \n
 *                                         1: Reset relay is reset when watchdog has run down \n
 * @param[in] useInterrupt               : Interrupt flag (not used -> set to 0)
 *
 * @return     0 : No Error \n
 *             1 : The device parameter is wrong \n
 *             2 : The timer/watchdog index parameter is wrong \n
 *             3 : The mode parameter is wrong \n
 *             4 : The time unit parameter is wrong \n
 *             5 : The time value parameter is wrong \n
 *             6 : The gate flag parameter is wrong \n
 *             7 : The gate level parameter is wrong \n
 *             8 : The hardware trigger flag parameter is wrong \n
 *             9 : The hardware trigger level parameter is wrong \n
 *             10 : The warning relay flag parameter is wrong \n
 *             11 : The warning relay delay time unit parameter is wrong \n
 *             12 : The warning relay delay value parameter is wrong \n
 *             13 : The reset relay flag parameter is wrong \n
 *             14 : The reset relay mode parameter is wrong \n
 *             15 : The interrupt flag parameter is wrong
 */
int i_PCIE040_TimerWatchdogInitAndStart (uint8_t boardId,
                                         uint8_t index,
                                         uint8_t mode,
                                         uint8_t timeUnit,
                                         uint8_t timeValue,
                                         uint8_t useGate,
                                         uint8_t gateLevel,
                                         uint8_t useHardwareTrigger,
                                         uint8_t hardwareTriggerLevel,
                                         uint8_t warningRelayMode,
                                         uint8_t warningRelayDelayTimeUnit,
                                         uint8_t warningRelayDelayTimeValue,
                                         uint8_t useResetRelay,
                                         uint8_t resetRelayMode,
                                         uint8_t useInterrupt);

//--------------------------------------------------------------------------------------

/** Permits to initialize the timer/watchdog of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] index                      : Timer/Watchdog index (0 to 6) \n
 * @param[in] mode                       : 0: WATCHDOG \n
 *                                         1: TIMER \n
 * @param[in] timeUnit                   : Time unit \n
 *                                         1: µs 2: ms 3: s 4: min \n
 * @param[in] timeValue                  : Time value (1 to 255)
 * @param[in] useGate                    : (Only available in timer mode) Define if the gate is used or not \n
 *                                         0: Gate not used \n
 *                                         1: Gate used \n
 * @param[in] gateLevel                  : (Only available in timer mode)Gate level selection \n
 *                                         1: If the hardware gate is enabled, it is active at "0" \n
 *                                         2: If the hardware gate is enabled, it is active at "1" \n
 * @param[in] useHardwareTrigger         : (Only available in watchdog mode) Define if the hardware trigger is used or not \n
 *                                         0: Hardware trigger not used \n
 *                                         1: Hardware trigger used \n
 * @param[in] hardwareTriggerLevel       : (Only available in watchdog mode) Hardware trigger level selection \n
 *                                         1: If the hardware trigger is used, it triggers from "1" to "0" \n
 *                                         2: If the hardware trigger is used, it triggers from "0" to "1" \n
 *                                         3: If the hardware trigger is used, it triggers from "0" to "1" and from "1" to "0" \n
 * @param[in] warningRelayMode            : (Only available in watchdog mode on watchdog 1 to 6) Define if the warning relay mode \n
 *                                         0 : Warning relay not used \n
 *                                         1 : warning mode 1 \n
 *                                             If index = 0, the warning relais goes high when the watchdog runs down and goes low after the specified delay\n
 *                                             If index > 0, the warning relais goes high when the watchdog runs down. It goes low when the watchdog is newly initialized\n
 *                                         2 : warning mode 2 \n
 *                                             The warning relais goes high when the watchdog starts and goes low when the watchdog runs down
 * @param[in] warningRelayDelayTimeUnit  : (Only available in watchdog mode) Delay time between the warning relay and the activation of the reset relay \n
 *                                         1: µs 2: ms 3: s \n
 * @param[in] warningRelayDelayTimeValue : (Only available in watchdog mode) Delay value (1 to 255) \n
 * @param[in] useResetRelay              : (Only available for the first watchdog) Define if the reset delay is used or not \n
 *                                         0: Reset relay not used \n
 *                                         1: Reset relay used \n
 * @param[in] resetRelayMode             : (Only available for the first watchdog) Define the reset relay mode \n
 *                                         0: Reset relay is set when watchdog has run down \n
 *                                         1: Reset relay is reset when watchdog has run down \n
 * @param[in] useInterrupt               : Interrupt flag (not used -> set to 0)
 *
 * @return     0 : No Error \n
 *             1 : The device parameter is wrong \n
 *             2 : The timer/watchdog index parameter is wrong \n
 *             3 : The mode parameter is wrong \n
 *             4 : The time unit parameter is wrong \n
 *             5 : The time value parameter is wrong \n
 *             6 : The gate flag parameter is wrong \n
 *             7 : The gate level parameter is wrong \n
 *             8 : The hardware trigger flag parameter is wrong \n
 *             9 : The hardware trigger level parameter is wrong \n
 *             10 : The warning relay flag parameter is wrong \n
 *             11 : The warning relay delay time unit parameter is wrong \n
 *             12 : The warning relay delay value parameter is wrong \n
 *             13 : The reset relay flag parameter is wrong \n
 *             14 : The reset relay mode parameter is wrong \n
 *             15 : The interrupt flag parameter is wrong
 */
int i_PCIE040_TimerWatchdogInit (uint8_t boardId,
                                  uint8_t index,
                                  uint8_t mode,
                                  uint8_t timeUnit,
                                  uint8_t timeValue,
                                  uint8_t useGate,
                                  uint8_t gateLevel,
                                  uint8_t useHardwareTrigger,
                                  uint8_t hardwareTriggerLevel,
                                  uint8_t warningRelayMode,
                                  uint8_t warningRelayDelayTimeUnit,
                                  uint8_t warningRelayDelayTimeValue,
                                  uint8_t useResetRelay,
                                  uint8_t resetRelayMode,
                                  uint8_t useInterrupt);

//--------------------------------------------------------------------------------------

/** Permits to start / stop the timer / watchdog of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] index                      : Timer/Watchdog index (0 to 6) \n
 * @param[in] cmd                        : 0: Stop \n
 *                                         1: start \n
 *
 * @return     0 : No Error \n
 *             1 : The device parameter is wrong \n
 *             2 : The timer/watchdog index parameter is wrong \n
 *             3 : Error by the start / stop of the timer/watchdog
 */
int i_PCIE040_TimerWatchdogStartStop (uint8_t boardId,
                                       uint8_t index,
                                       uint8_t cmd);

//--------------------------------------------------------------------------------------

/** Read the status and the value of the timer/watchdog of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] index                      : Timer/Watchdog index (0 to 6) \n
 * @param[out] value                     : Timer / Watchdog value \n
 * @param[out] status                    : Timer / Watchdog status \n
 *                                         0: Timer / Watchdog is running \n
 *                                         1: Timer / Watchdog has run down
 * @param[out] softwareTriggerStatus     : Software trigger status\n
 *                                         0: No software trigger occur \n
 *                                         1: Software trigger occur \n
 * @param[out] hHardwareTriggerStatus    : Hardware trigger status \n
 *                                         0: No hardware trigger occur \n
 *                                         1: Hardware trigger occur \n
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 *            2 : The timer/watchdog index parameter is wrong \n
 *            3 : Error by reading the status of the timer/watchdog
 */
int i_PCIE040_TimerWatchdogReadStatusAndValue (uint8_t boardId,
                                                uint8_t index,
                                                uint8_t *value,
                                                uint8_t *status,
                                                uint8_t *softwareTriggerStatus,
                                                uint8_t *hardwareTriggerStatus);

//--------------------------------------------------------------------------------------

/** Trigger the timer/watchdog of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] index                      : Timer/Watchdog index (0 to 6) \n
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 *            2 : The timer/watchdog index parameter is wrong \n
 *            3 : Error by trigger of the timer/watchdog
 */
int i_PCIE040_TimerWatchdogTrigger (uint8_t boardId,
                                     uint8_t index);

//--------------------------------------------------------------------------------------

/** Permits to start / stop all timer/watchdog of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] mode                       : 0: WATCHDOG \n
 *                                         1: TIMER \n
 * @param[in] cmd                        : 0: Stop \n
 *                                         1: start \n
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 *            2 : The mode parameter is wrong \n
 *            3 : Error by the start / stop the selected timers
 */
int i_PCIE040_TimerWatchdogAllStartStop (uint8_t boardId,
                                          uint8_t mode,
                                          uint8_t cmd);

//--------------------------------------------------------------------------------------

/** Trigger all timer/watchdog of the board xPCIe-040
 *
 * @param[in] boardId                    : Board ID \n
 * @param[in] mode                       : 0: WATCHDOG \n
 *                                         1: TIMER \n
 *
 * @return    0 : No Error \n
 *            1 : The device parameter is wrong \n
 *            2 : The mode parameter is wrong \n
 */
int i_PCIE040_TimerWatchdogAllTrigger (uint8_t boardId,
                                        uint8_t mode);

/* @} */

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#endif // __PCIE040_H__
